/***************************************************************************
                          font.cpp  -  description
                             -------------------
    begin                : Sun Jan 30 2000
    copyright            : (C) 2000 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/


#include "font_instance.h"
#include <SDL.h>
#include "fluff.h"
#include "loader.h"
#include <map>

Font_instance::Font_instance()
{
	font = NULL;
}

//--------------------------------------------------------------------------------------
Font_instance::~Font_instance()
{
	if ( font ) {
		TTF_CloseFont ( font );
    free ( name );
  }
}

//--------------------------------------------------------------------------------------
bool Font_instance::load(Loader *loader, char *font_name, int32 size_ )
{
	char font_file[100];
	sprintf(font_file,"fonts/ttf/%s.ttf",font_name);

	char name_[100];

  name = (char*) malloc ( strlen ( font_name ) + 1 );
  strcpy ( name , font_name );
	sprintf ( name_ , loader->getfilename(font_file) );
	font = TTF_OpenFont ( name_ , size_ );
	if ( !font ) {
		logmsg ( lt_error , "Unable to load TTF font %s, size %d" , font_name , size_ );
		return false;
	}
	logmsg ( lt_debug , "Loaded TTF font %s, size %d" , font_name , size_ );
	size = size_;
	return true;
}

//--------------------------------------------------------------------------------------
void Font_instance::get_text_geom(char *text , int *length , int *height)
{
  // we need to display this text to know his length
  SDL_Color black = { 0 , 0 , 0 , 0 };
  SDL_Surface *text_surf = TTF_RenderText_Solid ( font, text, black );
  if ( !text_surf ) {
    logmsg ( lt_warning , "Unable to render text %s, using font %s" , text , name );
    return;
  }
	*length = text_surf -> w;
  *height = text_surf -> h;
  SDL_FreeSurface ( text_surf );
}

//--------------------------------------------------------------------------------------
bool Font_instance::put_text(int xpos, int ypos, char *text, Mutable_raw_surface *target , bool direct_copy , int32 red , int32 green , int32 blue )
{
	SDL_Color white = { red, green, blue, 0 };

  // FIXME: support for other font rendering styles
  SDL_Surface *text_surf = TTF_RenderText_Solid ( font, text, white );
  if ( !text_surf ) {
    logmsg ( lt_warning , "Unable to render text %s, using font %s" , text , name );
    return false;
  }
  // logmsg ( lt_message , "Text %s rendered, font %s" , text , name );
  Graphic *tmpgr = new Graphic;
  tmpgr -> create ( text_surf );
  if ( direct_copy )
    tmpgr -> draw_alpha ( target , xpos , ypos );
  else
    tmpgr -> draw_copy ( target , xpos , ypos );

  // free that !!!
  SDL_FreeSurface ( text_surf );
	return true;
}
