/***************************************************************************
                          nil_client.cpp  -  description
                             -------------------
    begin                : Mon Nov 15 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/

#include "systemheaders.h"
#include "configuration.h"
#include "nil_client.h"
#include "fluff.h"
#include "loader.h"
#include "all.h"
#include "loader.h"
#include "controls.h"
#include "graphic_bar.h"
#include "icon_graph.h"
#include "font_instance.h"
#include "game_client.h"
#include "SDL.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"
#include "soundsystem.h"

int njoys;
SDL_Joystick **joys;

void run_client(Configuration &config)
{
	setmsgsource("C");
	logmsg(lt_message,"Running client");

	int sdl_flags = SDL_INIT_AUDIO | SDL_INIT_VIDEO | SDL_INIT_JOYSTICK;

	if (config.get_option_int("coredump"))
		sdl_flags |= SDL_INIT_NOPARACHUTE;
	int sdl_error = SDL_Init(sdl_flags);
	if (sdl_error < 0) {
		logmsg(lt_fatal,"Unable to initialize SDL: %i",sdl_error);
		return;
	}

	atexit(SDL_Quit);
	if ( TTF_Init() < 0 ) {
		fprintf(stderr, "Couldn't initialize TTF: %s\n",SDL_GetError());
		return;
	}
	atexit(TTF_Quit);

	Config_xy screen_size = config.get_option_xy("screen_size");
	int sdl_video_flags = SDL_HWSURFACE;
	if (config.get_option_int("fullscreen"))
		sdl_video_flags |= SDL_FULLSCREEN;
	SDL_Surface *screen = SDL_SetVideoMode(screen_size.x, screen_size.y, 32, sdl_video_flags);
	if (!screen) {
		logmsg(lt_fatal,"Couldn't set %ix%ix32 video mode: %s\n",screen_size.x, screen_size.y,SDL_GetError());
		return;
	}
	logmsg(lt_debug,"SDL initialized.\n");

	njoys=SDL_NumJoysticks();
	logmsg(lt_debug,"Found %i joysticks, opening them:", njoys);

	joys=new (SDL_Joystick *)[njoys];
	for(int i=0; i<njoys; ++i)
	  {
	    logmsg(lt_debug, "  opening %s", SDL_JoystickName(i));
	    joys[i]=SDL_JoystickOpen(i);
	  }

	logmsg(lt_debug,"Going to add: %s as sources",config.get_option("data"));
	Loader loader;
	loader.addsources(config.get_option("data"));
	
	logmsg(lt_warning , "If nil crashes just beyond this point, try sound=0 option" );
	if ( config.get_option_int("sound") ) {
		Mix_OpenAudio(22050, AUDIO_S16SYS, 2, 1024);
		int channels = Mix_AllocateChannels(48);
		logmsg(lt_debug,"Mixer initialized, %i channels.",channels);
	}
	Evtmapper evtmapper;
	evtmapper.init(&config);

	//initialize the single player client:
	Game_client game_client;
	Soundsystem soundsystem(&loader, &game_client);
	game_client.init(&loader, &soundsystem, &config);

	int join_error=game_client.join_game(config.get_option("server"),
					     config.get_option_int("server_port"),
					     config.get_option_int("local_players"),
					     config.get_option("player_name"),
					     config.get_option_int("player_color"));

	if (join_error) {
		logmsg(lt_fatal,"Unable to join the game (%i), giving up and going home",join_error);
		exit(-1);
	}

	int view_quality = config.get_option_int("view_quality");
	game_client.setgeom(0,0,screen_size.x,screen_size.y,view_quality);


	//for stats:
	game_client.smallfont = loader.fonts.getfont("arial",10);

	bool done = false;
	//Obj_player_data *player_data = NULL;

	Icon_graph traffic_graph;
	traffic_graph.init(10,20,50,20,PIXELA(128,128,0,6),PIXELA(255,0,0,8));

	float traffic_trend = 0;
	float last_traffic_sample = 0;
	float traffic_sum = 0;
	bool plot_traffic = false;

	Graphic *logo=loader.graphics.getgraphic("logos/nil_small.rgbf");

	bool screen_shoot = false;
	bool show_logo = true;
	int screen_shot_number = 0;
	bool death_cam_armed = false;
	bool death_cam_active = false;
	int anon_anounce=0;

  while (!done) {

		if (request_quit)
			done = true;

		if (anon_anounce-- <= 0) {
			anon_anounce = 30;
			logmsg(lt_debug,"Anon objects: %i, time:%.2f",game_client.clientworld.get_anon_objects_inuse(),game_client.clientworld.gettime());
		}

		//handle the SDL events:
		SDL_PumpEvents();
		SDL_Event event;
		while (SDL_PollEvent(&event)) {
			//handle the keyboard/joystick button events:
			if (event.type==SDL_KEYUP || event.type==SDL_KEYDOWN ||
			    event.type==SDL_JOYBUTTONDOWN ||
			    event.type==SDL_JOYBUTTONUP) {
				Nil_inputevent inputevent;
				inputevent.sdl_inputevent = &event;
				evtmapper.lookup(inputevent);

				int target = inputevent.cmdfunction & CMDFUNCTION::TARGET_MASK;

				switch (target) {
					case (CMDFUNCTION::PLAYER1): {
						game_client.handle_input(&inputevent, 0);
						break;
					}

					case (CMDFUNCTION::PLAYER2): {
					        game_client.handle_input(&inputevent, 1);
						break;
					}

					case (CMDFUNCTION::HOTKEY): {
						if (!inputevent.pressed) break;

						//handle gen purpose hotkeys here...
						int function = inputevent.cmdfunction & CMDFUNCTION::FUNCTION_MASK;
						switch (function) {

  						case (CMDFUNCTION::MENU) :{
              	done = true;
								break;
							}		
		
							case CMDFUNCTION::SCREEN_SHOOT :{
              	screen_shoot = true;
								break;
							}

							case CMDFUNCTION::TOGGLE_TRAFFIC : {
								plot_traffic = !plot_traffic;
								if (plot_traffic)
									logmsg(lt_debug,"plotting of traffic on (100%% is 128kb/s)");
								else
									logmsg(lt_debug,"plotting of traffic off");
								break;
							}

							case CMDFUNCTION::TOGGLE_SHADOW : {
								game_client.dropshadow = !game_client.dropshadow;
								if (game_client.dropshadow)
									logmsg(lt_message,"Dropshadow has been turned on");
								else
									logmsg(lt_message,"Dropshadow has been turned off");
								break;
							}

							case CMDFUNCTION::TOGGLE_SMOOTH : {
								game_client.smooth = !game_client.smooth;
								if (game_client.smooth)
									logmsg(lt_message,"Smoothing has been turned on");
								else
									logmsg(lt_message,"Smoothing has been turned off");
								break;
							}

							case CMDFUNCTION::TOGGLE_LOGO : {
								show_logo = !show_logo;
								if (show_logo)
									logmsg(lt_debug,"The nifty logo is back.");
								else
									logmsg(lt_debug,"The nifty logo now gone, why oh why?");

								break;
							}

							case CMDFUNCTION::TOGGLE_DEATHCAM : {
								death_cam_active = !death_cam_active;
								if (death_cam_active)
									logmsg(lt_debug,"The DeathCam(tm) is now active.");
								else
									logmsg(lt_debug,"The DeathCam(tm) is now off.");

								break;
							}
							
						} //switch (function) {
						break;
					} //case (CMDFUNCTION::HOTKEY): {
			
					default: {
						logmsg(lt_warning,"Key with invalid target: %x", inputevent.cmdfunction);
					}   	
				} //switch (target) {
	

				if (event.type==SDL_KEYDOWN) {
					//a couple of keys for debugging:
					switch (((SDL_KeyboardEvent *) inputevent.sdl_inputevent)->keysym.sym) {
						case SDLK_q: game_client.clientworld.dump_objects(); break;
    	    	case SDLK_w: game_client.clientworld.getscoreboard()->dump(); break;
            case SDLK_e: game_client.clientworld.dump_time(); break;
              //case SDLK_BACKQUOTE: logmsg ( lt_message , "Quote pressed" );
						default:{}
					}
				}
		
			} //if (event.type==SDL_KEYUP || event.type==SDL_KEYDOWN) {
			else if (event.type==SDL_JOYAXISMOTION)
			  {
			    SDL_JoyAxisEvent *evt=((SDL_JoyAxisEvent *) &event);
			    game_client.handle_joyaxis(evt->which,
						       evt->axis,
						       evt->value);
			  }

		} //while (SDL_PollEvent(&event)) {

		//    if (!player_data)	{
		//			player_data = (Obj_player_data *)game_client.clientworld.get_object(game_client.clientworld.getowner(),ot_player_data);
		//			if (!player_data) logmsg(lt_warning,"Couldn't find player_data");	
		//		}
		game_client.think();

		Mutable_raw_surface tmp;
		if (SDL_MUSTLOCK(screen)) {
			if ( SDL_LockSurface(screen) < 0 )
				logmsg(lt_error,"Unable to lock SDL surface");		
		}
		tmp.attach(screen_size.x,screen_size.y,(int32 *)screen->pixels);

		game_client.draw(&tmp);

		if (show_logo) {
			logo->draw_alpha(&tmp,tmp.get_xsize()-logo->get_xsize()-2,15);
		}

		//write fps:
		{
			char str[140];
			float tick_time = game_client.get_trend_tick_duration();
			if (tick_time == 0) tick_time = 1;
			float fps = 1/tick_time;
			sprintf(str,"fps: %.2f, %d %d",fps,game_client.clientworld.getobjcount(), game_client.clientworld.get_anon_objects_inuse());
      if (game_client.smallfont)
      game_client.smallfont->put_text(10,0,str,&tmp,true,255,255,255);
		}
		if (plot_traffic) {
			traffic_sum += game_client.last_update_size;
			float traffic_time = game_client.clientworld.gettime()- last_traffic_sample;
			if (traffic_time > 1) {
				traffic_trend += ((traffic_sum/traffic_time)-traffic_trend)/5;

				traffic_sum = 0;
				last_traffic_sample = game_client.clientworld.gettime();
	  	  traffic_graph.addsample(traffic_trend/(128*1024/8));
			}
			traffic_graph.draw(&tmp);
		}

		if (death_cam_active) {
			if (game_client.clientworld.time_of_death >= 0 && (game_client.clientworld.gettime() - game_client.clientworld.time_of_death) == 0)
				death_cam_armed = true;
	
			if (death_cam_armed && (game_client.clientworld.gettime() - game_client.clientworld.time_of_death) > 0.2) {
				screen_shoot = true;
				death_cam_armed = false;
			}
		} else
			death_cam_armed = false;
	
		if (screen_shoot) {
			screen_shoot = false;
			screen_shot_number++;
	
			char filename[100];
			sprintf(filename,"/tmp/nil_screenshot_%i.rgbf",screen_shot_number);
			logmsg(lt_message,"Dumping screenshot to: %s",filename);

			//Make sure that all the pixels are visible:
			int32 *pixels = tmp.get_pixels_write();
			for (int y=0;y<tmp.get_ysize();y++)
				for (int x=0;x<tmp.get_xsize();x++) {
					int32 *pixel = pixels + x+y*tmp.get_xsize();
					*pixel |= 0xff000000;
				}
			tmp.save_to_file(filename);
		}

		if ( SDL_MUSTLOCK(screen) ) {
			SDL_UnlockSurface(screen);
		}

		SDL_UpdateRect(screen, 0,0, screen_size.x, screen_size.y);
  }

	delete[] joys;

	return;
}
