/***************************************************************************
                          obj_bonus.cpp
                             -------------------
    begin                : Sun Jan 20 2002
    copyright            : (C) 2002 by Daniel Burrows
    email                : dburrows@debian.org
 ***************************************************************************/

#include "systemheaders.h"
#include "obj_bonus.h"

Obj_bonus::Obj_bonus(World *world_, int id) : Obj_base(world_, id)
{
}

// Try at most 20 times to place a bonus before giving up.
//
// FIXME: catalog the "open" areas with a floodsearch and be smarter?  How
//       much computation will that take?
void Obj_bonus::init(int xpos_, int ypos_, float timeout_)
{
  xpos=xpos_;
  ypos=ypos_;
  timeout=world->gettime()+timeout_;
}

// FIXME: need to check onground() and decide whether to call simple_update()
// based on that! -- simple_update() doesn't avoid bouncing.  (bug in
// newton_update?)
void Obj_bonus::server_think()
{
  Map_hit hit=update_simple(true);

  int playerid=dynamic_cast<Serverworld *>(world)->find_radius_object(xpos, ypos, getradius(), ot_player_avatar);

  if(playerid!=-1)
    {
      Obj_player_avatar *hitter=(Obj_player_avatar *) world->get_object(playerid, ot_player_avatar);

      logmsg(lt_debug, "Player picked up bonus");

      pickup(hitter);
    }

  switch(hit.hittype) {
  case ht_object:
    break;

  case ht_left:
  case ht_right:
  case ht_top:
  case ht_map:
    bogo_bounce();
    break;

  case ht_none:
    break;
  }

  if(curtime>=timeout)
    do_timeout();

  setdirty();
}

void Obj_bonus::do_timeout()
{
  bedone();
}

void Obj_bonus::serialize(Serializer *serializer)
{
  Obj_base::serialize(serializer);

  serializer->rw(timeout);
}
