/***************************************************************************
                          obj_bonus.h  -  a bonus that falls from the sky.
                             -------------------
    begin                : Sun Jan 20 2002
    copyright            : (C) 2002 by Daniel Burrows
    email                : dburrows@debian.org
 ***************************************************************************/

/***************************************************************************
 *   This program is free software under the GNU GPL                       *
 ***************************************************************************/

#ifndef OBJ_BONUS_H
#define OBJ_BONUS_H

#include "obj_base.h"

// Only provides the basic behavior of bonuses; the children should
// handle additional stuff.

class Obj_player_avatar;

class Obj_bonus:public Obj_base {
public:
  Obj_bonus(World *world_, int id);

  virtual Object_type gettype()=0;

  void init(int xpos_, int ypos_, float timeout_);

  // Child-overridden behaviors.

  // Called when a player picks the bonus up.
  virtual void pickup(Obj_player_avatar *who)=0;


  // Called to determine the lifetime of the bonus.  Default is a random
  // value between 30 and 120 seconds.

  // Default behavior is to vanish harmlessly.  Some bonuses might
  // do something else.
  virtual void do_timeout();

  // movement.
  void server_think();

  void serialize(Serializer *serializer);

  bool hurtable() {return true;}

  // Bonuses will "hit" anything within their radius.
  virtual float getradius()=0;
protected:
  // At what time the bonus should vanish.
  float timeout;
};

#endif

