/***************************************************************************
                          obj_scoreboard.cpp  -  description
                             -------------------
    begin                : Sun Dec 26 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/


#include "obj_scoreboard.h"
#include "obj_player_data.h"

//-----------------------------------------------------------------------------
Obj_scoreboard::Obj_scoreboard(World *world_, int id) : Obj_base(world_, id)
{
	scores_inuse = 0;
	frags_inuse = 0;
}

//-----------------------------------------------------------------------------
Obj_scoreboard::~Obj_scoreboard()
{
}
	
//-----------------------------------------------------------------------------
void Obj_scoreboard::serialize(Serializer *serializer)
{
	serializer->rw(scores_inuse);
	{ //MS-compiler bug workaround
		for(int i=0;i<scores_inuse;i++) {
			serializer->rw(scores[i].player_id);
			serializer->rw(scores[i].kills);
			serializer->rw(scores[i].deaths);
			serializer->rw(scores[i].frags);
		}
  }
	serializer->rw(frags_inuse);
	{
		for ( int i = 0; i < frags_inuse; ++i ) {
			serializer->rw ( last_killers [ i ] );
			serializer->rw ( last_victims [ i ] );
      serializer->rw ( keep_frag_time [ i ] );
		}
	}
	//put all the names last this is a small hack to get as much out of soft serializing as possible

	{ //MS-compiler bug workaround
		for(int i=0;i<scores_inuse;i++)
			serializer->rwstr(scores[i].playername,sizeof(scores[i].playername));
	}
}

//-----------------------------------------------------------------------------
void Obj_scoreboard::server_think ()
{
  int imax = 0;
  while (( imax < frags_inuse ) && ( keep_frag_time [ imax ] < world -> gettime() ))
    ++imax;
  if ( imax == 0 )
    return;
  //  logmsg ( lt_message , "%d frags deprecated" , imax );
  frags_inuse -= imax;
  for ( int i = frags_inuse - 1; i >= 0; --i ) {
    last_killers [ i ] = last_killers [ i + imax ];
    last_victims [ i ] = last_victims [ i + imax ];
    keep_frag_time [ i ] = keep_frag_time [ i + imax ];
  }
	setdirty ();
}

//-----------------------------------------------------------------------------
void Obj_scoreboard::addfrag(int killer_id, int victim_id)
{
	logmsg(lt_debug,"frag: %i %i",killer_id, victim_id);
	Obj_player_data *killer_player = (Obj_player_data *)world->get_object(killer_id,ot_player_data);
	Obj_player_data *victim_player = (Obj_player_data *)world->get_object(victim_id,ot_player_data);

	//First update the killer:
	//If this wasn't a guy killing himself, give the killer a frag
	{	
		int frag = killer_id==victim_id?-1:1;
		int i=get_index(killer_id);
		if (i >= 0) {
			if ( killer_id != victim_id )
				scores[i].kills++;
			scores[i].frags += frag;
			if (killer_player) strcpy(scores[i].playername,killer_player->get_player_name());
		} else {
			if (killer_player)
				strcpy(scores[scores_inuse].playername,killer_player->get_player_name());
			else
				strcpy(scores[scores_inuse].playername,"Unknown.");

			scores[scores_inuse].player_id = killer_id;
			if ( killer_id != victim_id )
				scores[scores_inuse].kills = 1;
      else
        scores[scores_inuse].kills = 0;
			scores[scores_inuse].deaths = 0;  	
			scores[scores_inuse].frags = frag;
			scores_inuse++;
		}
	}

	//Update the victim:
	{
		int i=get_index(victim_id);
		if (i >= 0) {
			scores[i].deaths++;
			if (victim_player) strcpy(scores[i].playername,victim_player->get_player_name());
		} else {
			if (victim_player)
				strcpy(scores[scores_inuse].playername,victim_player->get_player_name());
			else
				strcpy(scores[scores_inuse].playername,"Unknown.");

			scores[scores_inuse].player_id = victim_id;
			scores[scores_inuse].kills = 0;
			scores[scores_inuse].deaths = 1;  	
			scores[scores_inuse].frags = 0;
			scores_inuse++;
		}
	}
	// Update last few frags
	{
		if ( frags_inuse >= KEEP_FRAGS ) {
			for ( int i = 1; i < KEEP_FRAGS; ++i ) {
				last_killers [ i - 1 ] = last_killers [ i ];
				last_victims [ i - 1 ] = last_victims [ i ];
				keep_frag_time [ i - 1 ] = keep_frag_time [ i ];
			}
    } else
			frags_inuse++;
    last_killers [ frags_inuse - 1 ] = killer_id;
    last_victims [ frags_inuse - 1 ] = victim_id;
    keep_frag_time [ frags_inuse - 1 ] = KEEP_FRAG_TIME + world -> gettime ();
  }
	setdirty();
}

//-----------------------------------------------------------------------------
Obj_scoreboard::Score Obj_scoreboard::getscore(int index)
{
	int ind = get_index ( index );
	if (ind >= 0 && ind < scores_inuse) {
		return scores[ind];
	} else {
		Score s;
		s.player_id = -1;
		s.frags = 0;
		s.kills = 0;
		s.deaths = 0;
		return s;
	}
}
	
//-----------------------------------------------------------------------------
int Obj_scoreboard::get_index(int player_id)
{
	for(int i=0;i<scores_inuse;i++)
		if (scores[i].player_id == player_id)
			return i;
	return -1;
}

//-----------------------------------------------------------------------------
void Obj_scoreboard::dump()
{
	logmsg(lt_message,"%i scores:",scores_inuse);
	for(int i=0;i<scores_inuse;i++){
		Obj_player_data *pd = (Obj_player_data *)world->get_object(scores[i].player_id,ot_player_data);
		if (pd)
  		logmsg(lt_message,"%s: f:%2i k:%2i d:%2i",pd->get_player_name(),scores[i].frags,scores[i].kills,scores[i].deaths);
		else
  		logmsg(lt_message,"%s: f:%2i k:%2i d:%2i",scores[i].playername,scores[i].frags,scores[i].kills,scores[i].deaths);
	}
  logmsg ( lt_message , "frags_inuse: %d" , frags_inuse );
  if ( frags_inuse > 0 )
    for ( int i = 0; i < frags_inuse; ++i )
      logmsg ( lt_message , "last_kills: %d - %d at %f" , last_killers [ i ] , last_victims [ i ] , keep_frag_time [ i ] );
}
