#include <SDL.h>

#include "fluff.h"
#include "loader.h"
#include "graphic.h"
#include "raw_surface.h"

#include "configuration.h"

void view_rgbf(Configuration &config)
{
  int sdl_flags = /*SDL_INIT_AUDIO |*/ SDL_INIT_VIDEO;

  int sdl_error = SDL_Init(sdl_flags);
  if (sdl_error < 0) {
    logmsg(lt_fatal,"Unable to initialize SDL: %i",sdl_error);
    return;
  }

  atexit(SDL_Quit);

  Loader loader;
  loader.addsources(config.get_option("data"));

  Graphic *g=loader.graphics.getgraphic(strdup(config.get_option("inputfile")));

  int sdl_video_flags = SDL_HWSURFACE;

  SDL_Surface *screen = SDL_SetVideoMode(g->get_xsize(), g->get_ysize(), 32, sdl_video_flags);

  if (!screen) {
    logmsg(lt_fatal,"Couldn't set %ix%ix32 video mode: %s\n",g->get_xsize(),g->get_ysize(),SDL_GetError());
    return;
  }

  Mutable_raw_surface tmp;
  if (SDL_MUSTLOCK(screen)) {
    if ( SDL_LockSurface(screen) < 0 )
      logmsg(lt_error,"Unable to lock SDL surface");		
  }
  tmp.attach(g->get_xsize(),g->get_ysize(),(int32 *)screen->pixels);

  g->draw_copy_stretch(&tmp, 0, 0, g->get_xsize(), g->get_ysize(),
		       0, 0);

  if ( SDL_MUSTLOCK(screen) ) {
    SDL_UnlockSurface(screen);
  }

  SDL_UpdateRect(screen, 0,0, g->get_xsize(), g->get_ysize());

  bool done=false;

  while (!done) {
    SDL_PumpEvents();
    SDL_Event event;
    while (SDL_PollEvent(&event)) {
      if(event.type==SDL_KEYDOWN && ((SDL_KeyboardEvent *) &event)->keysym.sym==SDLK_q)
	done=true;
    }
  }
}
