/***************************************************************************
                            -  description
                             -------------------
    begin                : Tue Oct 12 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/

#ifndef WORLD_H
#define WORLD_H

class Obj_base;
class Obj_scoreboard;

#include "systemheaders.h"
#include "viewport.h"
#include "fluff.h"
#include "loader.h"
#include "graphic.h"
#include "graphics.h"

#define RNDTABLE_SIZE 15001

class Obj_player_avatar;
class World_objects;
class Soundsystem;

/*
 World knows about the map of the world, as well as the objects in it.
*/
class World {
	public:
	World(Loader *loader_, Soundsystem *soundsystem_, bool always_reload_);
	World();
	virtual ~World();
	void init(Loader *loader_, Soundsystem *soundsystem_);

	/*
	 * returns true if this world is a client
	 * it is important to know if we are in a client or a server when making up our minds about important stuff
	 */
	virtual bool isclient()=0;

	//in the server this will simply make a hole, on the client it will spawn some debris
	virtual void makehole(int x, int y, int diameter)=0;

	/*
	 * creates a blank map in the desired size
	 */
	void initmap(int width, int height);

	/*
	 * creates a filled map in the desired size
	 */
	void createmap(int width, int height, unsigned int randomseed_);

	/*
	 * creates a map from liero map file
	 */
	void createmap( char *map_filename );

	/* newobject is my factory that creates concrete objects
	 * Object_type objecttype	: The object type to spawn
   * char *initial_state		: A pointer to the objects first state info
	 * int state_size					:	The size of the state struct
	 * int objectid = -1			:	If you want to set the object id, this is the what you want on a client (not a server)
   * it will return the pointer to the new object
	 */
	Obj_base *newobject(Object_type objecttype, int objectid);

	//inline access functions:
	float gettime() 	{return curtime;}
	void settime(float newtime)
		{
			//remember the time for this frame for calculating the delta time later
			//NOTE: This will only work correctly if time is only set once pr frame...
			last_frame_time = curtime;
			curtime = newtime;
		}

  Loader *loader;
//	Graphics graphics;

	Mutable_raw_surface map_buffer;
	Mutable_raw_surface bgmap_buffer;

	//Simple and fast algo, it is also kind of inccurate +-radius, use for bombs and missiles.
	bool collision(int x0, int y0, int x1, int y1, int &xc, int &yc, int radius);
	
	//The same algo, however it calls is_passable on object to test for collision,
	//this is accurate, but somewhat slower, this was written for Obj_player in mind,
	//so speed is not that important here.
	bool collision(int x0, int y0, int x1, int y1, int &xc, int &yc, Obj_base *object);

	//These are a few sugar coated ways of getting to objects in the world.
	int getobjcount();
	//	Obj_base *get_object_by_index(int index);

	bool get_always_reload();

	Obj_base *get_object(int objectid, Object_type expected_type);
	Obj_player_avatar *get_player_avatar(int playerid);
	bool get_player_pos(int playerid, int &x, int &y);

	int World::get_player_count();
	Obj_player_avatar *World::get_player_number( int iIndex );
	
	float get_last_frame_delta() {return curtime-last_frame_time;}

	void dump_objects();

  void dump_time () { logmsg ( lt_message , "Now is %f AD" , gettime() ); }

	Obj_scoreboard *getscoreboard() {return (Obj_scoreboard *)get_object(scoreboard_id,ot_scoreboard);}

	//This is set byt avatars when they die
	float time_of_death;

	void assert_object_type(int objectid, Object_type expected_type);
	
	// Sound system
	Soundsystem *soundsystem;

	protected:
	Graphic *holes[16];
	void remove_dead_objects();

	//FIXME Make this an std::multimap (or hash_multimap?) instead:
	World_objects *objects;

	float last_frame_time; //used for calculating how much time the last frame took
	float curtime;

	//object id of the scoreboard object
	int scoreboard_id;

	//The randomseed used for the map:
	unsigned int randomseed;

	// If true, weapons are reloaded whenever possible; otherwise,
	// only a player's selected weapon is reloaded.
	bool always_reload;
};

#endif

